﻿using System;
using System.Collections.Generic;
using System.Data.Objects;
using System.Linq;
using AdventureWorks;


namespace CompiledQueries1 {
    internal class CompiledCustomerRepository : IDisposable {
        #region Sex enum

        public enum Sex {
            Man,
            Woman,
            Both
        }

        #endregion

        private static readonly Func<AdventureWorksLT2008_DataEntities, IQueryable<Customer>> AllCustomers =
            CompiledQuery.Compile<AdventureWorksLT2008_DataEntities, IQueryable<Customer>>(
                                                                                           db =>
                                                                                           db.Customer.Select(cus => cus));

        private static readonly Func<AdventureWorksLT2008_DataEntities, IQueryable<Customer>> AllMaleCustomers =
            CompiledQuery.Compile<AdventureWorksLT2008_DataEntities, IQueryable<Customer>>(
                                                                                           db =>
                                                                                           db.Customer.Where(cus => cus.Title == "Mr."));

        private static readonly Func<AdventureWorksLT2008_DataEntities, IQueryable<Customer>> AllFemaleCustomers =
            CompiledQuery.Compile<AdventureWorksLT2008_DataEntities, IQueryable<Customer>>(
                                                                                           db =>
                                                                                           db.Customer.Where(cus => cus.Title == "Ms."));

        private static readonly Func<AdventureWorksLT2008_DataEntities, string, IQueryable<Customer>> AllMaleCustomersStartingWith =
            CompiledQuery.Compile<AdventureWorksLT2008_DataEntities, string, IQueryable<Customer>>(
                                                                                            (db, start) =>
                                                                                            db.Customer.Where(cus =>
                                                                                                cus.Title == "Mr." &&
                                                                                                cus.LastName.StartsWith(start))
            );

        private static readonly Func<AdventureWorksLT2008_DataEntities, string, IQueryable<Customer>> AllFemaleCustomersStartingWith =
            CompiledQuery.Compile<AdventureWorksLT2008_DataEntities, string, IQueryable<Customer>>(
                                                                                            (db, start) =>
                                                                                            db.Customer.Where(cus =>
                                                                                                cus.Title == "Ms." &&
                                                                                                cus.LastName.StartsWith(start))
            );

        private static readonly Func<AdventureWorksLT2008_DataEntities, string, IQueryable<Customer>> AllCustomersStartingWith =
            CompiledQuery.Compile<AdventureWorksLT2008_DataEntities, string, IQueryable<Customer>>(
                                                                                            (db, start) =>
                                                                                            db.Customer.Where(cus => cus.LastName.StartsWith(start))
            );

        private readonly AdventureWorksLT2008_DataEntities _db;

        public CompiledCustomerRepository() {
            this._db = new AdventureWorksLT2008_DataEntities();
        }

        #region IDisposable Members

        public void Dispose() { if (this._db != null) this._db.Dispose(); }

        #endregion

        public List<Customer> GetCustomers() {
            return AllCustomers(this._db).ToList();
        }

        public List<Customer> GetCustomers(Sex sex) {
            List<Customer> result;

            switch (sex) {
                case Sex.Man:
                    result = AllMaleCustomers(this._db).ToList();
                    break;
                case Sex.Woman:
                    result = AllFemaleCustomers(this._db).ToList();
                    break;
                default:
                    result = AllCustomers(this._db).ToList();
                    break;
            }
            return result;
        }

        public List<Customer> GetCustomers(Sex sex, string lastNameStartsWith) {
            List<Customer> result;

            switch (sex) {
                case Sex.Man:
                    result = AllMaleCustomersStartingWith(this._db, lastNameStartsWith).ToList();
                    break;
                case Sex.Woman:
                    result = AllFemaleCustomersStartingWith(this._db, lastNameStartsWith).ToList();
                    break;
                default:
                    result = AllCustomersStartingWith(this._db, lastNameStartsWith).ToList();
                    break;
            }
            return result;
        }
    }
}