﻿using System;
using System.Linq;
using AdventureWorks;


namespace CompiledQueries1 {
    internal class CustomerRepository : IDisposable {
        #region Sex enum

        public enum Sex {
            Man,
            Woman,
            Both
        }

        #endregion

        private readonly AdventureWorksLT2008_DataEntities _db;

        public CustomerRepository() {
            this._db = new AdventureWorksLT2008_DataEntities();
        }

        #region IDisposable Members

        public void Dispose() {
            if (this._db != null)
                this._db.Dispose();
        }

        #endregion

        public IQueryable<Customer> GetCustomers() {
            return this._db.Customer.Select(cus => cus);
        }

        public IQueryable<Customer> GetCustomers(Sex sex) {
            IQueryable<Customer> result = this.GetCustomers();

            switch (sex) {
                case Sex.Man:
                    result = result.Where(cus => cus.Title == "Mr.");
                    break;
                case Sex.Woman:
                    result = result.Where(cus => cus.Title == "Ms.");
                    break;
            }
            return result;
        }

        public IQueryable<Customer> GetCustomers(Sex sex, string lastNameStartsWith) {
            return this.GetCustomers(sex).Where(cus => cus.LastName.StartsWith(lastNameStartsWith));
        }
    }
}